/////////////////////////////////////////////////////////////
// GAME STATE CLASS
//
class GS{
  // GLOBAL VARIABLES
  int difficulty = 0; //set how common blue plates are generated, with 0 being easiest/most common
  int life = 5; //set the number of hearts, with 1 being minimum
  int state = 0; //set game state, 0=title, 1=play, 2=dying, 3=lost, 4=won
  int[] fatstack = new int[20]; //set number of plates of food in order to win game
  // CONSTRUCTOR
  GS(){
  }
  //
  // FUNCTIONS
  //
  void run(){ //main function of the gameplay
    drawChair(width/4, 3*height/4+50); //draw Your chair
    drawChair(3*width/4, 3*height/4+50); //draw Fat Man's chair
    drawMan(3*width/4, 3*height/4); //draw Fat Man
    drawYou(width/4, 3*height/4+50); //draw You
    drawTable(); //draw Table
    checkfstate(); //check the state of the foods
    drawlife(); //draw the hearts on the screen
    drawstack(); //draw the stacked plates
  }
  //
  void chairBreaks(float n){ //when game.state = 2, the chair breaks down and Fat Man falls down
    drawChair(width/4, 3*height/4+50); //normally draw Your chair
    int x = 3*width/4;
    int y = 3*height/4+50;
    rectMode(CENTER);
    if (n < 90){ //this is to control the angle of the chair rotating
      fill(#552F1D);
      pushMatrix();
      translate(x-130,y-50);
      rotate(radians(-n));
      rect(0,0, 20,500); //draw chair
      popMatrix();
      pushMatrix();
      translate(x+130,y-50);
      rotate(radians(n));
      rect(0,0, 20,500); //draw chair
      popMatrix();
    }
    else{ //when the chair is rotated through 90 degrees, end the animation
      state = 3; //takes player to game over screen
    }
    fill(#B2273C);
    rect(x,y-200+2*n, 260,140); //draw chair
    drawMan(3*width/4, (3*height/4)+3*n); //draw Fat Man, but with a changing position and velocity
    drawManEat(3*width/4, (3*height/4)+3*n); //draw Fat Man's open mouth, but also moving with the body
    drawYou(width/4, 3*height/4+50); //draw You normally
    drawTable(); //draw Table normally
  }
  //
  void drawlife(){ //draw hearts on screen based on the life number
    for (int i = life-1; i >= 0; i--){
      image(img, width-50*(i+1), 0); //"-50" is to ensure they are evenly spaced out
    }
  }
  //
  void drawstack(){ //draw the stack of plates extracted from the int[] List
    int t = 20; //default plate position
    for (int i = fatstack.length-1; i >= 0; i--){
      if (fatstack[i] != 0){ //if the plate color isn't already filled
        fill(fatstack[i]); //plate color depending on the fatstack value
        pushMatrix();
        scale(2);
        ellipse((width-75)/2,((height-100)/2)-100+(t*5), 60,30); //plate
        t --; //subtract as each plate is stacked
        popMatrix();
      }
    }
  }
  //
  void replaceplate(int c){ //color the black plates to either blue or red
    for (int i = fatstack.length-1; i >= 0; i--){
      if (fatstack[i] == 0){ //if plate is black
        fatstack[i] = c; //color it with the parameter c
        break;
      }
    }    
  }
  //
  void checkgamestate(){ //check to see if the game is winning or losing
    if (life == 0){ //no more hearts
      game.state = 2; //switch to chair-breaking screen
    }
    if (fatstack[0] != 0){ //if stack of plates reaches max
      game.state = 4; //switch to winning screen
    }
  }
  //
  void checkfstate(){ //check the state of foods and move their positions
    if (Foods.size() <= 5){ //if there are less than 5 food in the ArrayList
      Food newfood = new Food(randomF(game.difficulty));
      if (Foods.size() != 0){
        Food lastfood = (Food) Foods.get(Foods.size()-1); //retrieve the last food in the ArrayList
        newfood.pos.x = lastfood.pos.x-30*3; //set the initial position of the new food, based off the last in the ArrayList
      }
      Foods.add(newfood); //add the new food into the ArrayList
    }
    for (int i = Foods.size()-1; i >= 0; i--){ //go through every Food in ArrayList
      Food food = (Food) Foods.get(i);
        food.vel.x = globalvel/2; //set velocity based on globalvel (/2 because it's scaled 2.0)
        food.pos.x += food.vel.x; //set position based on velocity
        food.drawFood(food.pos.x,food.pos.y); //draw food based on defined position
        if (food.pos.x > 3*width/8){ //if food position is past the Fat Man's eat line
          Foods.remove(i); //remove it
          food.eat(); //have Fat Man eat it
          break;
        }
    }
  }
  //
  void drawMan(int x, float y){
    fill(255);
    ellipse(x,y, 400,400); //body
    arc(x-100,y-100, 100,70, radians(70),radians(200)); //(his) right moob
    pushMatrix();
    translate(x+100,y-100);
    rotate(radians(180));
    arc(0,0, 100,70, radians(160),radians(300)); //(his) left moob
    popMatrix();
    fill(#F7EBC3);
    ellipse(x,y-200, 100,100); //chest
    ellipse(x,y-280, 200,200); //head
    pushMatrix();
    translate(x,y-260);
    rotate(radians(315));
    arc(0,0, 80,80, radians(70),radians(200)); //mouth
    popMatrix();
    arc(x+50,y-250, 50,50, radians(90),radians(180)); //(his) left cheek
    pushMatrix();
    translate(x-50,y-250);
    rotate(radians(270));
    arc(0,0, 50,50, radians(90),radians(180)); //(his) right cheek
    popMatrix();
    pushMatrix();
    translate(x,y-210);
    rotate(radians(315));
    arc(0,0, 40,40, radians(70),radians(200)); //chin
    popMatrix();
    fill(0);
    ellipse(x-48,y-300, 15,30);
    ellipse(x+48,y-300, 15,30); //eyes
  }
  //
  void drawManEat(int x, float y){
    fill(#F7EBC3);
    ellipse(x,y-280, 200,200); //head
    fill(255,0,0);
    ellipse(x,y-225, 80,30); //opened mouth
    fill(#F7EBC3);
    arc(x+50,y-250, 50,50, radians(90),radians(180)); //(his) left cheek
    pushMatrix();
    translate(x-50,y-250);
    rotate(radians(270));
    arc(0,0, 50,50, radians(90),radians(180)); //(his) right cheek
    popMatrix();
    pushMatrix();
    translate(x,y-205);
    rotate(radians(315));
    arc(0,0, 40,40, radians(70),radians(200)); //chin
    popMatrix();
        fill(0);
    ellipse(x-48,y-300, 15,30);
    ellipse(x+48,y-300, 15,30); //eyes
  }
  //
  void drawYou(int x, int y){
    fill(#6950AF);
    ellipse(x,y, 200, 400); //body
    fill(#F0DDA0);
    ellipse(x,y-200, 70,70); //chest
    ellipse(x,y-280, 200,200); //head
    pushMatrix();
    translate(x,y-260);
    rotate(radians(315));
    arc(0,0, 80,80, radians(70),radians(200)); //mouth
    popMatrix();
    fill(0);
    ellipse(x-48,y-300, 15,30);
    ellipse(x+48,y-300, 15,30); //eyes
  }
  //
  void drawChair(int x, int y){
    fill(#552F1D);
    rectMode(CENTER);
    rect(x-130,y-50, 20,500);
    rect(x+130,y-50, 20,500);
    fill(#B2273C);
    rect(x,y-200, 260,140);
  }
  //
  void drawTable(){
    fill(#552F1D);
    rectMode(CENTER);
    rect(width/2,4*height/5+50, width+1,200); //table
    fill(#505050);
    rect(width/2,480, width+1,100); //conveyor belt
  }
}
